@@ -706,7 +706,7 @@ static BOOL expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunI
             if(*key == 0) {
                 char buf[128];
                 snprintf(buf, 128, "need hash key(%s)", env->mName);
-                err_msg(buf, runinfo->mSName, runinfo->mSLine);
+                err_msg(buf, runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 FREE(key);
                 return FALSE;
             }
@@ -715,7 +715,7 @@ static BOOL expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunI
                 if(object == NULL) {
                     char buf[128];
                     snprintf(buf, 128, "invalid key(%s)", key);
-                    err_msg(buf, runinfo->mSName, runinfo->mSLine);
+                    err_msg(buf, runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     FREE(key);
                     return FALSE;
                 }
@@ -727,7 +727,7 @@ static BOOL expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunI
         default: {
             char buf[128];
             snprintf(buf, 128, "can't expand variable because of type(%s)", env->mName);
-            err_msg(buf, runinfo->mSName, runinfo->mSLine);
+            err_msg(buf, runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             FREE(key);
             return FALSE;
             }
@@ -774,7 +774,7 @@ static BOOL expand_env_to_command(ALLOC char** result, char* str, sCommand* comm
                         FREE(buf.mBuf);
                         char buf[128];
                         snprintf(buf, 128, "no such as object(%s)", env->mName);
-                        err_msg(buf, runinfo->mSName, runinfo->mSLine);
+                        err_msg(buf, runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
                 }
@@ -792,6 +792,7 @@ static BOOL expand_env_to_command(ALLOC char** result, char* str, sCommand* comm
 
                 if(!run(env->mBlock, nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject))
                 {
+                    err_msg("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     runinfo->mRCode = rcode;
                     FREE(buf.mBuf);
                     return FALSE;
@@ -818,7 +819,7 @@ static BOOL get_option(MANAGED char* arg, sCommand* command, sObject* fun, sObje
             FREE(arg);
             char buf[BUFSIZ];
             snprintf(buf, BUFSIZ, "invalid option with argument(%s)", arg);
-            err_msg(buf, runinfo->mSName, runinfo->mSLine);
+            err_msg(buf, runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             return FALSE;
         }
 
@@ -833,7 +834,7 @@ static BOOL get_option(MANAGED char* arg, sCommand* command, sObject* fun, sObje
             {
                 char buf[BUFSIZ];
                 snprintf(buf, BUFSIZ, "option number max (%s)\n", arg);
-                err_msg(buf, runinfo->mSName, runinfo->mSLine);
+                err_msg(buf, runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 FREE(expanded_str);
                 FREE(arg);
                 return FALSE;
@@ -844,7 +845,7 @@ static BOOL get_option(MANAGED char* arg, sCommand* command, sObject* fun, sObje
             {
                 char buf[BUFSIZ];
                 snprintf(buf, BUFSIZ, "option number max (%s)\n", arg);
-                err_msg(buf, runinfo->mSName, runinfo->mSLine);
+                err_msg(buf, runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 FREE(arg);
                 return FALSE;
             }
@@ -855,7 +856,7 @@ static BOOL get_option(MANAGED char* arg, sCommand* command, sObject* fun, sObje
         if(!sCommand_put_option(command, MANAGED arg)) {
             char buf[BUFSIZ];
             snprintf(buf, BUFSIZ, "option number max (%s)\n", arg);
-            err_msg(buf, runinfo->mSName, runinfo->mSLine);
+            err_msg(buf, runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             FREE(arg);
             return FALSE;
         }
@@ -990,6 +991,9 @@ sObject* fun_new_from_gc(sObject* parent, BOOL user_object)
 
    SFUN(self).mArgBlocks = NULL;
 
+   SFUN(self).mRunInfo = NULL;
+
+
    return self;
 }
 
@@ -1008,6 +1012,8 @@ sObject* fun_new_from_stack(sObject* parent)
    SFUN(self).mLocalObjects = NULL;
    SFUN(self).mArgBlocks = NULL;
 
+   SFUN(self).mRunInfo = NULL;
+
    return self;
 }
 
@@ -1020,6 +1026,8 @@ sObject* fun_clone_from_stack_block_gc(sObject* block, BOOL user_object, sObject
    SFUN(self).mLocalObjects = NULL;
    SFUN(self).mArgBlocks = NULL;
 
+   SFUN(self).mRunInfo = NULL;
+
    return self;
 }
 
@@ -1032,6 +1040,8 @@ sObject* fun_clone_from_stack_block_stack(sObject* block, sObject* parent)
    SFUN(self).mLocalObjects = NULL;
    SFUN(self).mArgBlocks = NULL;
 
+   SFUN(self).mRunInfo = NULL;
+
    return self;
 }
 
