@@ -47,6 +47,8 @@ BOOL cmd_each(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         char* argument;
         if(argument = sCommand_option_with_argument_item(command, "-number")) {
+            runinfo->mRCode = 0;
+
             int number = atoi(argument);
 
             sObject* nextin2 = FD_NEW_STACK();
@@ -59,7 +61,7 @@ BOOL cmd_each(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 for(j=0; j<number && i+j<vector_size(SFD(nextin).mLines); j++)
                 {
                     if(!fd_write(nextin2, vector_item(SFD(nextin).mLines, i+j))) {
-                        err_msg("interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
@@ -67,17 +69,16 @@ BOOL cmd_each(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                 int rcode = 0;
                 if(!run(command->mBlocks[0], nextin2, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
-                    if(rcode == RCODE_BREAK) {
-                        break;
-                    }
-                    else {
-                        runinfo->mRCode = rcode;
-                        return FALSE;
-                    }
+                    runinfo->mRCode = rcode;
+                    return FALSE;
                 }
+
+                runinfo->mRCode = rcode;
             }
         }
         else {
+            runinfo->mRCode = 0;
+
             sObject* nextin2 = FD_NEW_STACK();
 
             int i;
@@ -85,25 +86,20 @@ BOOL cmd_each(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 fd_clear(nextin2);
 
                 if(!fd_write(nextin2, vector_item(SFD(nextin).mLines, i))) {
-                    err_msg("interrupt", runinfo->mSName, runinfo->mSLine);
+                    err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
                 }
 
                 int rcode = 0;
                 if(!run(command->mBlocks[0], nextin2, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
-                    if(rcode == RCODE_BREAK) {
-                        break;
-                    }
-                    else {
-                        runinfo->mRCode = rcode;
-                        return FALSE;
-                    }
+                    runinfo->mRCode = rcode;
+                    return FALSE;
                 }
+
+                runinfo->mRCode = rcode;
             }
         }
-
-        runinfo->mRCode = 0;
     }
 
     return TRUE;
@@ -144,12 +140,12 @@ BOOL cmd_join(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 sObject* line = STRING_NEW_STACK(vector_item(SFD(nextin).mLines, i));
                 string_chomp(line);
                 if(!fd_write(nextout, string_c_str(line))) {
-                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
                 }
                 if(!fd_write(nextout, field)) {
-                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
                 }
@@ -158,13 +154,13 @@ BOOL cmd_join(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             sObject* line = STRING_NEW_STACK(vector_item(SFD(nextin).mLines, i));
             string_chomp(line);
             if(!fd_write(nextout, string_c_str(line))) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
 
             if(!fd_write(nextout, "\n")) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
@@ -195,6 +191,8 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     }
 
     if(command->mArgsNumRuntime > 1 && command->mBlocksNum == command->mArgsNumRuntime-1) {
+        runinfo->mRCode = 0;
+
         fd_split(nextin, lf);
 
         if(vector_size(SFD(nextin).mLines) > 0) {
@@ -215,7 +213,7 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         buf2[len2] = 0;
                     }
                     else {
-                        err_msg("invalid range", runinfo->mSName, runinfo->mSLine);
+                        err_msg("invalid range", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
 
@@ -247,7 +245,7 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                             fd_clear(nextin2);
 
                             if(!fd_write(nextin2, vector_item(SFD(nextin).mLines, j))) {
-                                err_msg("interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
@@ -257,6 +255,8 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                                 runinfo->mRCode = rcode;
                                 return FALSE;
                             }
+
+                            runinfo->mRCode = rcode;
                         }
                     }
                     else {
@@ -267,7 +267,7 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                             fd_clear(nextin2);
 
                             if(!fd_write(nextin2, vector_item(SFD(nextin).mLines, j))) {
-                                err_msg("interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
@@ -277,6 +277,8 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                                 runinfo->mRCode = rcode;
                                 return FALSE;
                             }
+
+                            runinfo->mRCode = rcode;
                         }
                     }
                 }
@@ -295,7 +297,7 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     sObject* nextin2 = FD_NEW_STACK();
 
                     if(!fd_write(nextin2, vector_item(SFD(nextin).mLines, num))) {
-                        err_msg("interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
@@ -305,11 +307,11 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         runinfo->mRCode = rcode;
                         return FALSE;
                     }
+
+                    runinfo->mRCode = rcode;
                 }
             }
         }
-
-        runinfo->mRCode = 0;
     }
 
     return TRUE;
@@ -510,13 +512,14 @@ BOOL cmd_sort(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 for(i=0; i<vector_size(SFD(nextin).mLines); i++) {
                     char* line = vector_item(SFD(nextin).mLines, i);
                     if(!fd_write(nextout, line)) {
-                        err_msg("interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
                 }
-                runinfo->mRCode = 0;
             }
+
+            runinfo->mRCode = 0;
         } 
         else if(command->mBlocksNum == 1) {
             fd_split(nextin, lf);
@@ -531,13 +534,14 @@ BOOL cmd_sort(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 for(i=0; i<vector_size(SFD(nextin).mLines); i++) {
                     char* line = vector_item(SFD(nextin).mLines, i);
                     if(!fd_write(nextout, line)) {
-                        err_msg("interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
                 }
-                runinfo->mRCode = 0;
             }
+
+            runinfo->mRCode = 0;
         }
     }
 
