@@ -26,19 +26,21 @@
 
 BOOL cmd_jobs(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
+    sCommand* command = runinfo->mCommand;
+
     char buf[BUFSIZ];
     int i;
     for(i=0; i<vector_size(gJobs); i++) {
         sObject* job = vector_item(gJobs, i);
         snprintf(buf, BUFSIZ, "[%d] name: %s pid&pgrp: %d", i+1, SJOB(job).mName, SJOB(job).mPGroup);
         if(!fd_write(nextout, buf)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
 
         if(!fd_write(nextout, "\n")) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
@@ -79,7 +81,7 @@ BOOL cmd_exit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     }
     else {
         if(vector_size(gJobs) > 0) {
-            err_msg("jobs exist", runinfo->mSName, runinfo->mSLine);
+            err_msg("jobs exist", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             return FALSE;
         }
 
@@ -93,6 +95,8 @@ BOOL cmd_while(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sCommand* command = runinfo->mCommand;
     if(command->mBlocksNum == 2) {
+        runinfo->mRCode = 0;
+
         while(1) {
             /// condition ///
             int rcode;
@@ -108,16 +112,16 @@ BOOL cmd_while(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             /// inner loop ///
             if(!run(command->mBlocks[1], nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
                 if(rcode == RCODE_BREAK) {
-                    break;
+                    rcode = 0;
                 }
                 else {
                     runinfo->mRCode = rcode;
                     return FALSE;
                 }
             }
-        }
 
-        runinfo->mRCode = 0;
+            runinfo->mRCode = rcode;
+        }
     }
 
     return TRUE;
@@ -147,6 +151,8 @@ BOOL cmd_if(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
     if(command->mBlocksNum > 1) {
         if(command->mBlocksNum % 2 == 0) {
+            runinfo->mRCode = 0;
+
             int i;
             for(i=0; i<command->mBlocksNum; i+=2) {
                 /// a condition ///
@@ -163,11 +169,14 @@ BOOL cmd_if(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         runinfo->mRCode = rcode;
                         return FALSE;
                     }
+                    runinfo->mRCode = rcode;
                     break;
                 }
             }
         }
         else {
+            runinfo->mRCode = 0;
+
             BOOL run_else = TRUE;
 
             int i;
@@ -188,6 +197,7 @@ BOOL cmd_if(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         runinfo->mRCode = rcode;
                         return FALSE;
                     }
+                    runinfo->mRCode = rcode;
                     break;
                 }
             }
@@ -199,10 +209,10 @@ BOOL cmd_if(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     runinfo->mRCode = rcode;
                     return FALSE;
                 }
+
+                runinfo->mRCode = rcode;
             }
         }
-
-        runinfo->mRCode = 0;
     }
 
     return TRUE;
@@ -216,12 +226,14 @@ BOOL cmd_return(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
 BOOL cmd_stackinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
+    sCommand* command = runinfo->mCommand;
+
     char buf[BUFSIZ];
 
     snprintf(buf, BUFSIZ, "slot size %d\npool size %d\n", stack_slot_size(), stack_page_size());
 
     if(!fd_write(nextout, buf)) {
-        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         return FALSE;
     }
@@ -232,6 +244,7 @@ BOOL cmd_stackinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
 BOOL cmd_stackframe(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
+    sCommand* command = runinfo->mCommand;
     char buf[BUFSIZ];
 
     hash_it* it = hash_loop_begin(SFUN(runinfo->mRunningObject).mLocalObjects);
@@ -240,12 +253,12 @@ BOOL cmd_stackframe(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         char* key = hash_loop_key(it);
 
         char*obj_kind[16] = {
-            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "self matching completion"
+            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "external object"
         };
         snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[TYPE(object2)]);
 
         if(!fd_write(nextout, buf)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
@@ -257,14 +270,70 @@ BOOL cmd_stackframe(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+BOOL cmd_funinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sObject* running_object = runinfo->mRunningObject;
+
+    if(running_object && (TYPE(running_object) == T_FUN || TYPE(running_object) == T_CLASS) && SFUN(running_object).mRunInfo) {
+        sRunInfo* runinfo2 = SFUN(running_object).mRunInfo;
+
+        char buf[BUFSIZ];
+        sObject* object;
+
+        char current_object[128];
+        *current_object = 0;
+        object = runinfo2->mCurrentObject;
+        while(object && object != gRootObject) {
+            strcat(current_object, SUOBJECT(object).mName);
+            strcat(current_object, "<-");
+            object = SUOBJECT(object).mParent;
+        }
+        strcat(current_object, "root");
+
+        char reciever_object[128];
+        *reciever_object = 0;
+        object = runinfo2->mRecieverObject;
+        while(object && object != gRootObject) {
+            strcat(reciever_object, SUOBJECT(object).mName);
+            strcat(reciever_object, "<-");
+            object = SUOBJECT(object).mParent;
+        }
+        strcat(reciever_object, "root");
+
+        sCommand* command = runinfo2->mCommand;
+
+        char command_name[128];
+        *command_name = 0;
+        int i = 0;
+        while(i < command->mMessagesNum) {
+            strcat(command_name, command->mMessages[i++]);
+            strcat(command_name, "->");
+        }
+        strcat(command_name, command->mArgs[0]);
+
+        snprintf(buf, BUFSIZ, "source name: %s\nsource line: %d\nrun nest level: %d\ncurrent object: %s\nreciever object: %s\ncommand name: %s\n", runinfo2->mSName, runinfo2->mSLine, runinfo2->mNestLevel, current_object, reciever_object, command_name);
+
+        if(!fd_write(nextout, buf)) {
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+            return FALSE;
+        }
+
+        runinfo->mRCode = 0;
+    }
+
+    return TRUE;
+}
+
 BOOL cmd_gcinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
+    sCommand* command = runinfo->mCommand;
     char buf[BUFSIZ];
 
     snprintf(buf, BUFSIZ, "free objects %d\nused objects %d\nall object slot %d\nslot size %d\npool size %d\n", gc_free_objects_num(), gc_used_objects_num(), gc_slot_size()*gc_pool_size(), gc_slot_size(), gc_pool_size());
 
     if(!fd_write(nextout, buf)) {
-        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         return FALSE;
     }
@@ -300,7 +369,7 @@ BOOL cmd_sweep(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         snprintf(buf, BUFSIZ, "%d objects deleted\n", num);
 
         if(!fd_write(nextout, buf)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
@@ -314,9 +383,11 @@ BOOL cmd_sweep(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             sObject* item = hash_item(current_object, var_name);
 
-            if(item->mUserObject) {
-                if(hash_erase(current_object, var_name)) {
-                    runinfo->mRCode = 0;
+            if(item && item->mUserObject) {
+                if(item != gCurrentObject || strcmp(var_name, SUOBJECT(item).mName) != 0) {
+                    if(hash_erase(current_object, var_name)) {
+                        runinfo->mRCode = 0;
+                    }
                 }
             }
         }
@@ -341,18 +412,18 @@ BOOL cmd_print(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         int i;
         for(i=1; i<command->mArgsNumRuntime-1; i++) {
             if(!fd_write(out, command->mArgsRuntime[i])) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
             if(!fd_write(out, " ")) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
         }
         if(!fd_write(out, command->mArgsRuntime[i])) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
@@ -361,7 +432,7 @@ BOOL cmd_print(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     else if(runinfo->mFilter) {
         if(sCommand_option_item(command, "-read-from-error")) {
             if(!fd_write(out, SFD(gStderr).mBuf)) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
@@ -369,7 +440,7 @@ BOOL cmd_print(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         }
         else {
             if(!fd_write(out, SFD(nextin).mBuf)) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
@@ -400,10 +471,11 @@ BOOL cmd_eval(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     if(command->mBlocksNum > 0) {
         int rcode;
         if(!run(command->mBlocks[0], nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
+            err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = rcode;
             return FALSE;
         }
-        runinfo->mRCode = 0;
+        runinfo->mRCode = rcode;
     }
     else if(command->mArgsNumRuntime >= 2) {
         char* buf = command->mArgsRuntime[1];
@@ -416,19 +488,20 @@ BOOL cmd_eval(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         snprintf(buf2, BUFSIZ, "%s %d: eval", runinfo->mSName, runinfo->mSLine);
         if(parse(buf, buf2, &sline, block, NULL)) {
             if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
+                err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = rcode;
                 stack_end_stack();
                 return FALSE;
             }
         }
         else {
-            err_msg_adding("parser error\n", runinfo->mSName, runinfo->mSLine);
+            err_msg_adding("parser error\n", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             stack_end_stack();
             return FALSE;
         }
 
         stack_end_stack();
-        runinfo->mRCode = 0;
+        runinfo->mRCode = rcode;
     }
     else if(runinfo->mFilter) {
         char* buf = SFD(nextin).mBuf;
@@ -441,19 +514,20 @@ BOOL cmd_eval(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         snprintf(buf2, BUFSIZ, "%s %d: eval", runinfo->mSName, runinfo->mSLine);
         if(parse(buf, buf2, &sline, block, NULL)) {
             if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
+                err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = rcode;
                 stack_end_stack();
                 return FALSE;
             }
         }
         else {
-            err_msg_adding("parser error\n", runinfo->mSName, runinfo->mSLine);
+            err_msg_adding("parser error\n", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             stack_end_stack();
             return FALSE;
         }
 
         stack_end_stack();
-        runinfo->mRCode = 0;
+        runinfo->mRCode = rcode;
     }
 
     return TRUE;
@@ -473,13 +547,13 @@ BOOL cmd_msleep(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             snprintf(buf, 32, "%c\033[D", c[i % 3]);
 
             if(!fd_write(nextout, buf)) {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
             if(nextout == gStdout) {
                 if(!fd_flash(nextout, 1)) {
-                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
                 }
@@ -499,7 +573,7 @@ BOOL cmd_raise(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     sCommand* command = runinfo->mCommand;
 
     if(command->mArgsNumRuntime == 2) {
-        err_msg(command->mArgsRuntime[1], runinfo->mSName, runinfo->mSLine);
+        err_msg(command->mArgsRuntime[1], runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         return FALSE;
     }
 
@@ -520,7 +594,7 @@ BOOL cmd_time(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             runinfo->mRCode = rcode;
             return FALSE;
         }
-        runinfo->mRCode = 0;
+        runinfo->mRCode = rcode;
 
         time_t timer2;
         time(&timer2);
@@ -533,7 +607,7 @@ BOOL cmd_time(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         snprintf(buf, 1024, "%d sec(%d minuts %d sec)\n", sec, minuts, sec2);
 
         if(!fd_write(nextout, buf) ) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
@@ -583,7 +657,7 @@ static BOOL pipes_seek_external_programs(char* path, sObject* sys, char* sname,
     else {
         char buf[BUFSIZ];
         snprintf(buf, BUFSIZ, "%s is invalid path", path);
-        err_msg(buf, sname, sline);
+        err_msg(buf, sname, sline, "rehash");
         return FALSE;
     }
 
@@ -594,7 +668,7 @@ BOOL pipes_rehash(char* sname, int sline)
 {
     char* path = getenv("PATH");
     if(path == NULL) {
-        err_msg("PATH environment variables is NULL", sname, sline);
+        err_msg("PATH environment variables is NULL", sname, sline, "rehash");
         return FALSE;
     }
 
@@ -663,13 +737,13 @@ BOOL cmd_try(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     return FALSE;
                 }
                 else {
-                    runinfo->mRCode = 0;
+                    runinfo->mRCode = rcode;
                     return TRUE;
                 }
             }
         }
         else {
-            runinfo->mRCode = 0;
+            runinfo->mRCode = rcode;
             return TRUE;
         }
     }
@@ -679,8 +753,9 @@ BOOL cmd_try(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
 BOOL cmd_errmsg(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
+    sCommand* command = runinfo->mCommand;
     if(!fd_write(nextout, string_c_str(gErrMsg))) {
-        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         return FALSE;
     }
@@ -715,7 +790,7 @@ BOOL cmd_block(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             if(!fd_write(nextout, buf))
             {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
@@ -736,7 +811,7 @@ BOOL cmd_block(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             sObject* block = vector_item(SFUN(fun).mArgBlocks, block_num);
             sObject* fun = fun_clone_from_stack_block_stack(block, NULL);
 
-            if(!run_function(fun, runinfo->mCurrentObject, nextin, nextout, runinfo, command->mArgsRuntime + 2, command->mArgsNumRuntime -2, command->mBlocks, command->mBlocksNum))
+            if(!run_function(fun, nextin, nextout, runinfo, command->mArgsRuntime + 2, command->mArgsNumRuntime -2, command->mBlocks, command->mBlocksNum))
             {
                 return FALSE;
             }
@@ -758,7 +833,7 @@ BOOL cmd_block(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             if(!fd_write(nextout, SBLOCK(block).mSource))
             {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
