@@ -686,7 +686,7 @@ BOOL cmd_condition_re(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         }
         else {
             onig_free(reg);
-            err_msg("=~: invalid regex", runinfo->mSName, runinfo->mSLine);
+            err_msg("=~: invalid regex", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             return FALSE;
         }
     }
