@@ -26,7 +26,9 @@
 
 BOOL cmd_mshow(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
-    sObject* object = runinfo->mCurrentObject;
+    sCommand* command = runinfo->mCommand;
+
+    sObject* object = runinfo->mRecieverObject;
     hash_it* it = hash_loop_begin(object);
     while(it) {
         sObject* object2 = hash_loop_item(it);
@@ -34,12 +36,12 @@ BOOL cmd_mshow(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         char buf[BUFSIZ];
         char*obj_kind[16] = {
-            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "self matching completion"
+            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "external object"
         };
         snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[TYPE(object2)]);
 
         if(!fd_write(nextout, buf)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
@@ -65,7 +67,7 @@ static BOOL entry_fun(sObject* nextin, sObject* nextout, sRunInfo* runinfo, sObj
         sObject* parent = hash_item(object, command->mArgsRuntime[1]);
         if(parent) {
             if(SFUN(parent).mParentCount > PIPES_INHERIT_MAX) {
-                err_msg("can't inherit a parent function because of a parent function number limit", runinfo->mSName, runinfo->mSLine);
+                err_msg("can't inherit a parent function because of a parent function number limit", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 return FALSE;
             }
 
@@ -74,12 +76,12 @@ static BOOL entry_fun(sObject* nextin, sObject* nextout, sRunInfo* runinfo, sObj
                 SFUN(block).mParentCount = SFUN(parent).mParentCount + 1;
             }
             else {
-                err_msg("can't inherit a parent function beacuse of the different type", runinfo->mSName, runinfo->mSLine);
+                err_msg("can't inherit a parent function beacuse of the different type", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 return FALSE;
             }
         }
         else {
-            err_msg("can't inherit. There is not a parent", runinfo->mSName, runinfo->mSLine);
+            err_msg("can't inherit. There is not a parent", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             return FALSE;
         }
     }
@@ -98,7 +100,7 @@ static BOOL entry_fun(sObject* nextin, sObject* nextout, sRunInfo* runinfo, sObj
                 *p = 0;
                 if(strlen(option) > 1) {
                     if(!fun_put_option_with_argument(block, STRDUP(option))) {
-                        err_msg("option number is full.", runinfo->mSName, runinfo->mSLine);
+                        err_msg("option number is full.", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
                 }
@@ -115,7 +117,7 @@ static BOOL entry_fun(sObject* nextin, sObject* nextout, sRunInfo* runinfo, sObj
                     }
                 }
                 else {
-                    err_msg("invalid option name", runinfo->mSName, runinfo->mSLine);
+                    err_msg("invalid option name", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     return FALSE;
                 }
             }
@@ -123,7 +125,7 @@ static BOOL entry_fun(sObject* nextin, sObject* nextout, sRunInfo* runinfo, sObj
         *p = 0;
         if(strlen(option) > 1) {
             if(!fun_put_option_with_argument(block, STRDUP(option))) {
-                err_msg("option number is full.", runinfo->mSName, runinfo->mSLine);
+                err_msg("option number is full.", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 return FALSE;
             }
         }
@@ -154,7 +156,7 @@ BOOL cmd_def(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             sObject* block = BLOCK_NEW_STACK();
             int sline = 1;
             if(!parse(SFD(nextin).mBuf, "def", &sline, block, NULL)) {
-                err_msg_adding("parser error", runinfo->mSName, runinfo->mSLine);
+                err_msg("parser error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 stack_end_stack();
                 return FALSE;
             }
@@ -174,13 +176,13 @@ BOOL cmd_def(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             if(fun && TYPE(fun) == T_FUN) {
                 char* source = SBLOCK(fun).mSource;
                 if(!fd_write(nextout, source)) {
-                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
                 }
                 if(source[strlen(source)-1] != '\n') {   /// pomch
                     if(!fd_write(nextout, "\n")) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
@@ -188,7 +190,7 @@ BOOL cmd_def(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 runinfo->mRCode = 0;
             }
             else {
-                err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
+                err_msg("not found variable", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 return FALSE;
             }
         }
@@ -217,7 +219,7 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             sObject* block = BLOCK_NEW_STACK();
             int sline = 1;
             if(!parse(SFD(nextin).mBuf, "def", &sline, block, NULL)) {
-                err_msg_adding("parser error", runinfo->mSName, runinfo->mSLine);
+                err_msg("parser error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 stack_end_stack();
                 return FALSE;
             }
@@ -237,13 +239,13 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             if(klass && TYPE(klass) == T_CLASS) {
                 char* source = SBLOCK(klass).mSource;
                 if(!fd_write(nextout, source)) {
-                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
                 }
                 if(source[strlen(source)-1] != '\n') {   /// pomch
                     if(!fd_write(nextout, "\n")) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
@@ -251,7 +253,7 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 runinfo->mRCode = 0;
             }
             else {
-                err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
+                err_msg("not found variable", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 return FALSE;
             }
         }
@@ -262,6 +264,8 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
 BOOL cmd_inherit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
+    sCommand* command = runinfo->mCommand;
+
     sObject* running_object = runinfo->mRunningObject;
     if(runinfo->mRunningObject) {
         switch(TYPE(running_object)) {
@@ -283,13 +287,13 @@ BOOL cmd_inherit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         return FALSE;
                     }
                     if(runinfo->mRCode == RCODE_NFUN_INVALID_USSING) {
-                        err_msg("invalid command using", runinfo->mSName, runinfo->mSLine);
+                        err_msg("invalid command using", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
                 }
                 else {  // T_FUN
                     sCommand* command = runinfo->mCommand;
-                    if(!run_function(parent, runinfo->mCurrentObject, nextin, nextout, runinfo, command->mArgsRuntime + 1, command->mArgsNumRuntime -1, command->mBlocks, command->mBlocksNum))
+                    if(!run_function(parent, nextin, nextout, runinfo, command->mArgsRuntime + 1, command->mArgsNumRuntime -1, command->mBlocks, command->mBlocksNum))
                     {
                         return FALSE;
                     }
@@ -306,7 +310,7 @@ BOOL cmd_inherit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             if(parent) {
                 sCommand* command = runinfo->mCommand;
-                if(!run_function(parent, runinfo->mCurrentObject, nextin, nextout, runinfo, command->mArgsRuntime + 1, command->mArgsNumRuntime -1, command->mBlocks, command->mBlocksNum))
+                if(!run_function(parent, nextin, nextout, runinfo, command->mArgsRuntime + 1, command->mArgsNumRuntime -1, command->mBlocks, command->mBlocksNum))
                 {
                     return FALSE;
                 }
@@ -321,7 +325,7 @@ BOOL cmd_inherit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         case T_NFUN:
         case T_BLOCK:
         case T_COMPLETION:
-            err_msg("There is not a parent object", runinfo->mSName, runinfo->mSLine);
+            err_msg("There is not a parent object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             return FALSE;
 
         default:
@@ -397,12 +401,12 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                             char* pos = str_kanjipos2pointer(code, string_c_str(var), num);
                             char* pos2 = str_kanjipos2pointer(code, pos, 1);
                             if(!fd_write3(nextout, pos, pos2-pos)) {
-                                err_msg("interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
                             if(!fd_write(nextout, field)) {
-                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
@@ -411,7 +415,7 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         runinfo->mRCode = 0;
                     }
                     else {
-                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
+                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
                 }
@@ -424,19 +428,19 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     sObject* var = access_object(command->mArgsRuntime[i], &reciever, runinfo->mRunningObject);
                     if(var && TYPE(var) == T_STRING) {
                         if(!fd_write(nextout, string_c_str(var))) {
-                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                             return FALSE;
                         }
                         if(!fd_write(nextout, field)) {
-                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                             return FALSE;
                         }
                         runinfo->mRCode = 0;
                     }
                     else {
-                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
+                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
                 }
@@ -471,7 +475,7 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             if(sCommand_option_item(command, "-shift")) {
                 for(;i-1<max; i++) {
                     if(!fd_write(nextout, vector_item(SFD(nextin).mLines, i-1))) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
@@ -504,7 +508,7 @@ BOOL cmd_object(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     return FALSE;
                 }
 
-                runinfo->mRCode = 0;
+                runinfo->mRCode = rcode;
             }
         }
         else {
@@ -535,19 +539,20 @@ BOOL cmd_mrun(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             return FALSE;
         }
 
-        runinfo->mRCode = 0;
+        runinfo->mRCode = rcode;
     }
     else if(command->mArgsNumRuntime >= 2) {
         sObject* current = runinfo->mCurrentObject;
         sObject* klass = access_object(command->mArgsRuntime[1], &current, runinfo->mRunningObject);
 
         if(klass && TYPE(klass) == T_CLASS) {
-            if(!run_function(klass, runinfo->mRecieverObject, nextin, nextout, runinfo, command->mArgsRuntime + 2, command->mArgsNumRuntime -2, command->mBlocks, command->mBlocksNum)) {
+            runinfo->mCurrentObject = runinfo->mRecieverObject;
+            if(!run_function(klass, nextin, nextout, runinfo, command->mArgsRuntime + 2, command->mArgsNumRuntime -2, command->mBlocks, command->mBlocksNum)) {
                 return FALSE;
             }
         }
         else {
-            err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
+            err_msg("not found variable", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             return FALSE;
         }
 
@@ -564,12 +569,12 @@ BOOL cmd_pwo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     sObject* object = gCurrentObject;
     while(object != gRootObject) {
         if(!fd_write(nextout, SUOBJECT(object).mName)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
         if(!fd_write(nextout, "<-")) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
@@ -578,13 +583,13 @@ BOOL cmd_pwo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     }
 
     if(!fd_write(nextout, SUOBJECT(object).mName)) {
-        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         return FALSE;
     }
 
     if(!fd_write(nextout, "\n")) {
-        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         return FALSE;
     }
@@ -604,7 +609,7 @@ BOOL cmd_co(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             runinfo->mRCode = 0;
         }
         else {
-            err_msg("can't change the current object", runinfo->mSName, runinfo->mSLine);
+            err_msg("can't change the current object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             return FALSE;
         }
     }
@@ -646,19 +651,19 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     char buf[128];
                     snprintf(buf, 128, "%p", item);
                     if(!fd_write(nextout, buf)) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
                     if(!fd_write(nextout, field)) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
                     runinfo->mRCode = 0;
                 }
                 else {
-                    err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
+                    err_msg("not found variable", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     return FALSE;
                 }
             }
@@ -684,19 +689,19 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 if(type >= 0) {
                     char buf[128];
                     char* obj_kind[16] = {
-                        NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "self matching completion"
+                        NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "external object"
                     };
                     snprintf(buf, 128, "%s\n", obj_kind[type]);
 
                     if(!fd_write(nextout, buf)) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
                     runinfo->mRCode = 0;
                 }
                 else {
-                    err_msg("invalid adress. it's not memory managed by a pipes gabage collecter", runinfo->mSName, runinfo->mSLine);
+                    err_msg("invalid adress. it's not memory managed by a pipes gabage collecter", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     return FALSE;
                 }
             }
@@ -719,13 +724,13 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                             runinfo->mRCode = 0;
                         }
                         else {
-                            err_msg("invalid adress. it's not memory managed by a pipes gabage collecter", runinfo->mSName, runinfo->mSLine);
+                            err_msg("invalid adress. it's not memory managed by a pipes gabage collecter", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                             return FALSE;
                         }
                     }
                     else if(shift) {
                         if(!fd_write(nextout, vector_item(SFD(nextin).mLines, i))) {
-                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                             return FALSE;
                         }
@@ -778,7 +783,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     snprintf(buf, 128, "%d\n", vector_size(ary));
 
                     if(!fd_write(nextout, buf)) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
@@ -786,7 +791,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     runinfo->mRCode = 0;
                 }
                 else {
-                    err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
+                    err_msg("not found variable", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     return FALSE;
                 }
             }
@@ -819,12 +824,12 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                             sObject* var = vector_item(ary, num);
 
                             if(!fd_write(nextout, string_c_str(var))) {
-                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
                             if(!fd_write(nextout, field)) {
-                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
@@ -833,7 +838,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         runinfo->mRCode = 0;
                     }
                     else {
-                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
+                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
                 }
@@ -850,13 +855,13 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                             sObject* var = vector_item(ary, j);
 
                             if(!fd_write(nextout, string_c_str(var))) {
-                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
 
                             if(!fd_write(nextout, field)) {
-                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
@@ -864,7 +869,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         runinfo->mRCode = 0;
                     }
                     else {
-                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
+                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
                 }
@@ -882,7 +887,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         char* argument;
         if(ary && (argument = sCommand_option_with_argument_item(command, "-append"))) {
             if(TYPE(ary) != T_VECTOR) {
-                err_msg("different type of object", runinfo->mSName, runinfo->mSLine);
+                err_msg("different type of object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 return FALSE;
             }
 
@@ -966,7 +971,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     snprintf(buf, 128, "%d\n", hash_count(hash));
 
                     if(!fd_write(nextout, buf)) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
@@ -974,7 +979,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     runinfo->mRCode = 0;
                 }
                 else {
-                    err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
+                    err_msg("not found variable", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     return FALSE;
                 }
             }
@@ -997,12 +1002,12 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                         if(var) {
                             if(!fd_write(nextout, string_c_str(var))) {
-                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
                             if(!fd_write(nextout, field)) {
-                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
@@ -1011,7 +1016,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         runinfo->mRCode = 0;
                     }
                     else {
-                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
+                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
                 }
@@ -1029,22 +1034,22 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                             sObject* var = hash_loop_item(it);
 
                             if(!fd_write(nextout, key)) {
-                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
                             if(!fd_write(nextout, field)) {
-                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
                             if(!fd_write(nextout, string_c_str(var))) {
-                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
                             if(!fd_write(nextout, field)) {
-                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
@@ -1054,7 +1059,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         runinfo->mRCode = 0;
                     }
                     else {
-                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
+                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
                 }
@@ -1071,7 +1076,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         /// number
         if(hash && sCommand_option_item(command, "-append")) {
             if(TYPE(hash) != T_HASH) {
-                err_msg("different type of object", runinfo->mSName, runinfo->mSLine);
+                err_msg("different type of object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 return FALSE;
             }
 
@@ -1155,12 +1160,12 @@ BOOL cmd_export(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 char* env = getenv(command->mArgsRuntime[i]);
                 if(env) {
                     if(!fd_write(nextout, env)) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
                     if(!fd_write(nextout, field)) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
@@ -1191,7 +1196,7 @@ BOOL cmd_export(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             if(sCommand_option_item(command, "-shift")) {
                 for(;i-1<max; i++) {
                     if(!fd_write(nextout, vector_item(SFD(nextin).mLines, i-1))) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
@@ -1215,7 +1220,7 @@ BOOL cmd_unset(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         for(i=1; i<command->mArgsNumRuntime; i++) {
             char* name = command->mArgsRuntime[i];
             if(unsetenv(name) != 0) {
-                err_msg("invalid env name", runinfo->mSName, runinfo->mSLine);
+                err_msg("invalid env name", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 return FALSE;
             }
         }
