@@ -70,6 +70,10 @@ static void object_delete(sObject* obj)
     case T_MEMCHECKER:
         fprintf(stderr, "unexpected err on object_delete at gc.c");
         exit(1);
+
+    case T_EXTOBJ:
+        SEXTOBJ(obj).mFreeFun(SEXTOBJ(obj).mObject);
+        break;
     }
 }
 
@@ -110,6 +114,10 @@ int object_gc_children_mark(sObject* self)
     case T_COMPLETION:
         count += block_gc_children_mark(self);
         break;
+
+    case T_EXTOBJ:
+        count += SEXTOBJ(self).mMarkFun(self);
+        break;
     }
 
     return count;
