@@ -76,7 +76,7 @@ static BOOL read_quote(char**p, sBuf* buf, char* sname, int* sline)
 
     switch(**p) {
         case 0:
-            err_msg("unexpected end. can't quote null.", sname, *sline);
+            err_msg("unexpected end. can't quote null.", sname, *sline, buf->mBuf);
             return FALSE;
 
         case '\n':
@@ -141,7 +141,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
                         lf = kCRLF;
                     }
                     else {
-                        err_msg("invalid linefield", sname, *sline);
+                        err_msg("invalid linefield", sname, *sline, buf->mBuf);
                         return FALSE;
                     }
                 }
@@ -156,7 +156,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
                 break;
             
             default:
-                err_msg("invalid linefield", sname, *sline);
+                err_msg("invalid linefield", sname, *sline, buf->mBuf);
                 return FALSE;
         }
     }
@@ -203,7 +203,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
 
             while(1) {
                 if(**p == 0) {
-                    err_msg("close } which is used by expand variable", sname, *sline);
+                    err_msg("close } which is used by expand variable", sname, *sline, name.mBuf);
                     FREE(name.mBuf);
                     FREE(key.mBuf);
                     return FALSE;
@@ -243,7 +243,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
 
             skip_spaces(p);
             if(**p != ']') {
-                err_msg("require ] character", sname, *sline);
+                err_msg("require ] character", sname, *sline, name.mBuf);
                 FREE(name.mBuf);
                 FREE(key.mBuf);
                 return FALSE;
@@ -315,7 +315,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
         }
         else if(squote || dquote) {
             if(**p == 0) {
-                err_msg("close single quote or double quote", sname, *sline);
+                err_msg("close single quote or double quote", sname, *sline, buf->mBuf);
                 return FALSE;
             }
             else if(**p == '\n') {
@@ -370,7 +370,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
                     (*p)++;
                 }
                 else {
-                    err_msg("invalid here document name", sname, *sline);
+                    err_msg("invalid here document name", sname, *sline, buf->mBuf);
                     return FALSE;
                 }
             }
@@ -382,12 +382,12 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
                 (*p)++;
             }
             else {
-                err_msg("invalid here document name", sname, *sline);
+                err_msg("invalid here document name", sname, *sline, buf->mBuf);
                 return FALSE;
             }
 
             if(string_c_str(name)[0] == 0) {
-                err_msg("invalid here document name", sname, *sline);
+                err_msg("invalid here document name", sname, *sline, buf->mBuf);
                 return FALSE;
             }
 
@@ -430,7 +430,8 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
             }
         }
         /// option ///
-        else if(command->mArgsNum > 0 && buf->mLen == 0 && **p == '-' && (*(*p+1) < '0' || *(*p+1) > '9')) {
+        else if(command->mArgsNum > 0 && buf->mLen == 0 && **p == '-' && (isalpha(*(*p+1)) || *(*p+1) == '-' || *(*p+1) == '_'))
+        {
             add_char_to_buf(buf, PARSER_MAGIC_NUMBER_OPTION);
             (*p)++;
         }
@@ -451,7 +452,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
             buf->mGlob = TRUE;
         }
         /// message passing ///
-        else if(**p == '-' && *(*p+1) == '>') {
+        else if(command->mArgsNum == 0 && **p == '-' && *(*p+1) == '>') {
             (*p)+=2;
             buf->mMessagePassing = TRUE;
             SBLOCK(block).mCompletionFlags &= ~(COMPLETION_FLAGS_NEXT_STATMENT|COMPLETION_FLAGS_NEXT_PIPE);
@@ -472,7 +473,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
     }
 
     if(squote || dquote) {
-        err_msg("require to close ' or \"", sname, *sline);
+        err_msg("require to close ' or \"", sname, *sline, buf->mBuf);
         return FALSE;
     }
 
@@ -554,7 +555,7 @@ static BOOL glob_expasion(sBuf* buf, sCommand* command, char* sname, int* sline)
     }
 
     if(rc == GLOB_NOSPACE) {
-        err_msg("read_one_argument: out of space during glob operation", sname, *sline);
+        err_msg("read_one_argument: out of space during glob operation", sname, *sline, buf->mBuf);
         return FALSE;
     }
     else if(rc == GLOB_NOMATCH) {
@@ -604,7 +605,7 @@ static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObje
         }
 
         if(**p == 0) {
-            err_msg("close block. require ) character before the end of statments", sname, *sline);
+            err_msg("close block. require ) character before the end of statments", sname, *sline, NULL);
             return FALSE;
         }
     }
@@ -623,6 +624,8 @@ static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObje
 static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object)
 {
     while(**p) {
+        SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_WORDBREAK_CHARACTER;
+
         sBuf buf;
         memset(&buf, 0, sizeof(sBuf));
         buf.mBuf = MALLOC(64);
@@ -647,7 +650,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
         /// message passing ///
         else if(buf.mMessagePassing) {
             if(!sCommand_add_message(command, MANAGED buf.mBuf)) {
-                err_msg("too many message", sname, *sline);
+                err_msg("too many message", sname, *sline, buf.mBuf);
                 FREE(buf.mBuf);
                 return FALSE;
             }
@@ -746,7 +749,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
         }
         /// spaces ///
         else if(**p == ' ' || **p == '\t') {
-            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_FILE_COMPLETION;
+            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_WORDBREAK_CHARACTER;
             skip_spaces(p);
         }
         else {
@@ -787,11 +790,11 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
 
     while(**p) {
         if(statment->mCommandsNum >= STATMENT_COMMANDS_MAX) {
-            err_msg("There are max number of commands on this statment", sname, *sline);
+            err_msg("There are max number of commands on this statment", sname, *sline, NULL);
             return FALSE;
         }
 
-        SBLOCK(block).mCompletionFlags &= ~(COMPLETION_FLAGS_FILE_COMPLETION|COMPLETION_FLAGS_NEXT_PIPE);
+        SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_NEXT_PIPE;
 
         sCommand* command = sCommand_new(statment);
         if(!read_command(p, command, statment, block, sname, sline, current_object)) {
@@ -802,7 +805,7 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
         if(command->mArgsNum == 0)
         {
             if(command->mMessagesNum > 0) {
-                err_msg("require command name", sname, *sline);
+                err_msg("require command name", sname, *sline, NULL);
                 return FALSE;
             }
             else {
@@ -882,7 +885,7 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
     else {
         char buf[128];
         snprintf(buf, 128, "unexpected token3 -->(%c)\n", **p);
-        err_msg(buf, sname, *sline);
+        err_msg(buf, sname, *sline, NULL);
         return FALSE;
     }
 
@@ -922,7 +925,7 @@ BOOL parse(char* p, char* sname, int* sline, sObject* block, sObject** current_o
         if(*p == ')') {
             char buf[128];
             snprintf(buf, 128, "unexpected token3 -->(%c)\n", *p);
-            err_msg(buf, sname, *sline);
+            err_msg(buf, sname, *sline, NULL);
             return FALSE;
         }
         /// skip comment ///
