@@ -217,7 +217,7 @@ typedef struct {
     int mFlags;
 } sStatment;
 
-#define COMPLETION_FLAGS_FILE_COMPLETION 0x0100
+#define COMPLETION_FLAGS_WORDBREAK_CHARACTER 0x0100
 #define COMPLETION_FLAGS_NEXT_PIPE 0x0200
 #define COMPLETION_FLAGS_NEXT_STATMENT 0x0400
 #define COMPLETION_FLAGS_ENV 0x0800
@@ -235,6 +235,7 @@ struct block_obj
 };
 
 struct _sObject;
+struct _sRunInfo;
 
 typedef struct                // a function object can be treated as block_obj
 {
@@ -250,9 +251,9 @@ typedef struct                // a function object can be treated as block_obj
     struct _sObject* mArgBlocks;
 
     option_hash_it mOptions[PIPES_OPTION_MAX];
+    struct _sRunInfo* mRunInfo;
 } fun_obj;
 
-struct _sRunInfo;
 typedef BOOL (*fNativeFun)(struct _sObject*, struct _sObject*, struct _sRunInfo*);
 
 typedef struct 
@@ -301,6 +302,14 @@ typedef struct {
    int mCounter;
 } memchecker_obj;
 
+typedef int (*fExtObjMarkFun)(struct _sObject* self);
+typedef void (*fExtObjFreeFun)(void* self);
+
+typedef struct {
+    void* mObject;
+    fExtObjMarkFun mMarkFun;
+    fExtObjFreeFun mFreeFun;
+} external_obj;
 
 #define GC_MARK 0x80000000
 
@@ -318,6 +327,7 @@ typedef struct {
 #define T_CLASS 12
 #define T_EXTERNAL 13
 #define T_COMPLETION 14
+#define T_EXTOBJ 15
 
 typedef struct _sObject {
     int mFlg;                         // contains a kind of the above and a mark flag
@@ -338,6 +348,7 @@ typedef struct _sObject {
         nfun_obj uNativeFun;
         fun_obj uFunction;
         external_prog_obj uExternalProg;
+        external_obj uExternalObj;
     };
 } sObject;
 
@@ -355,6 +366,7 @@ typedef struct _sObject {
 #define SMEMCHECKER(o) (o)->uMemchecker
 #define SFUN(o) (o)->uFunction
 #define SEXTERNAL(o) (o)->uExternalProg
+#define SEXTOBJ(o) (o)->uExternalObj
 
 void gc_init(int pool_size);
 void gc_final();
@@ -377,6 +389,9 @@ int stack_page_size();
 #include "block.h"
 #include "curses.h"
 
+sObject* extobj_new_from_gc(void* object, fExtObjMarkFun mark_fun, fExtObjFreeFun free_fun, BOOL user_object);
+#define EXTOBJ_NEW_GC(o, o2, o3, o4) extobj_new_from_gc(o, o2, o3, o4)
+
 sObject* uobject_new_from_gc(int size, sObject* parent, char* name, BOOL user_object);
 sObject* uobject_new_from_stack(int size, sObject* object, char* name);
 void uobject_init(sObject* self);
@@ -423,7 +438,13 @@ BOOL pipes_rehash(char* sname, int sline);
 #if defined(HAVE_MIGEMO_H)
 BOOL cmd_migemo_match(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 #endif
+BOOL cmd_funinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_readline_point_move(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_readline(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_readline_clear_screen(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_readline_insert_text(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_fselector(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_readline_delete_text(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_mshow(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_mrun(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_errmsg(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
@@ -627,8 +648,8 @@ void pipes_readline_interface(char** argv, int argc, BOOL no_runtime_script);
 
 void fd_split(sObject* self, enum eLineField lf);
 
-void err_msg(char* msg, char* sname, int line);
-void err_msg_adding(char* msg, char* sname, int line);
+void err_msg(char* msg, char* sname, int line, char* command);
+void err_msg_adding(char* msg, char* sname, int line, char* command);
 
 sObject* memchecker_new_from_stack(int size);
 void memchecker_delete_stack(sObject* self);
@@ -680,8 +701,9 @@ extern FILE* gTest;
 
 sObject* access_object(char* name, sObject** current, sObject* running_object);
 sObject* access_object2(char* name, sObject* current, sObject* running_object);
+sObject* access_object3(char* name, sObject** current);
 
-BOOL run_function(sObject* fun, sObject* current_object, sObject* nextin, sObject* nextout, sRunInfo* runinfo, char** argv, int argc, sObject** blocks, int blocks_num);
+BOOL run_function(sObject* fun, sObject* nextin, sObject* nextout, sRunInfo* runinfo, char** argv, int argc, sObject** blocks, int blocks_num);
 
 extern sObject* gDirStack;
 
