@@ -82,10 +82,20 @@ static char* message_completion(const char* text, int stat)
 
                     sObject* candidate = STRING_NEW_STACK(string_c_str(messages));
                     string_push_back(candidate, key);
+
                     if(TYPE(item) == T_UOBJECT) {
-                        string_push_back(candidate, "->");
+                        vector_add(gCompletionArray, string_c_str(candidate));
+
+                        sObject* candidate2 = STRING_NEW_STACK(string_c_str(messages));
+                        string_push_back(candidate2, key);
+                        string_push_back(candidate2, "->");
+
+                        vector_add(gCompletionArray, string_c_str(candidate2));
                     }
-                    vector_add(gCompletionArray, string_c_str(candidate));
+                    else {
+                        vector_add(gCompletionArray, string_c_str(candidate));
+                    }
+
                     it = hash_loop_next(it);
                 }
 
@@ -136,10 +146,18 @@ static char* program_completion(const char* text, int stat)
 
                     sObject* candidate = STRING_NEW_STACK(hash_loop_key(it));
                     if(TYPE(object) == T_UOBJECT) {
-                        string_push_back(candidate, "->");
+                        if(hash_item(object, "main")) {
+                            vector_add(gCompletionArray, string_c_str(candidate));
+                        }
+                        else {
+                            sObject* candidate2 = STRING_NEW_STACK(hash_loop_key(it));
+                            string_push_back(candidate2, "->");
+                            vector_add(gCompletionArray, string_c_str(candidate2));
+                        }
+                    }
+                    else {
+                        vector_add(gCompletionArray, string_c_str(candidate));
                     }
-
-                    vector_add(gCompletionArray, string_c_str(candidate));
                 }
 
                 it = hash_loop_next(it);
@@ -174,9 +192,50 @@ static char* program_completion(const char* text, int stat)
     return NULL;
 }
 
+static void make_text_myself(char* text2, char* text, int size)
+{
+    char* p = text;
+    char* p2 = text2;
+    BOOL squote = FALSE;
+    BOOL dquote = FALSE;
+    while(*p) {
+        if(!squote && *p == '"') {
+            p++;
+            dquote = !dquote;
+        }
+        else if(!dquote && *p =='\'') {
+            p++;
+            squote = !squote;
+        }
+        else if(squote || dquote) {
+            if(p2 - text2 + 1 < size) {
+                *p2++ = *p++;
+            }
+        }
+        else if(*p == '\\') {
+            if(p2 - text2 + 1< size) {
+                *p2++ = *p++;
+                *p2++ = *p++;
+            }
+        }
+        else if(*p == ' ' || *p == '\t') {
+            p++;
+            p2 = text2;
+        }
+        else {
+            if(p2 - text2 + 1< size) {
+                *p2++ = *p++;
+            }
+        }
+    }
+
+    *p2 = 0;
+}
+
 static char* user_completion(const char* text, int stat)
 {
     static int index, wordlen;
+    static char text2[1024];
 
     if(stat == 0) {
         gCompletionArray = VECTOR_NEW_STACK(16);
@@ -190,7 +249,10 @@ static char* user_completion(const char* text, int stat)
 
         vector_sort(gCompletionArray, name_sort);
 
-        wordlen = strlen(text);
+        make_text_myself(text2, rl_line_buffer, 1024);
+//printf("text2 (%s) rl_line_buffer (%s)\n", text2, rl_line_buffer);
+
+        wordlen = strlen(text2);
         index = 0;
     }
 
@@ -198,7 +260,7 @@ static char* user_completion(const char* text, int stat)
         char* candidate = vector_item(gCompletionArray, index);
         index++;
 
-        if(!strncmp(text, candidate, wordlen)) {
+        if(!strncmp(text2, candidate, wordlen)) {
             if(candidate[strlen(candidate)-1] == '/') {
                 rl_completion_append_character = 0;
             }
@@ -344,6 +406,29 @@ static char* env_completion(const char* text, int stat)
     return NULL;
 }
 
+static void get_current_completion_object(sObject** completion_object, sObject** current_object)
+{
+    if(*current_object != gRootObject) {
+        sObject* parent_object = SUOBJECT(*current_object).mParent;
+        get_current_completion_object(completion_object, &parent_object);
+        if(*completion_object) *completion_object = hash_item(*completion_object, SUOBJECT(*current_object).mName);
+    }
+}
+
+static sObject* access_object_compl(char* name, sObject** current)
+{
+    sObject* object;
+
+    while(1) {
+        object = hash_item(*current, name);
+
+        if(object || *current == gCompletionObject) { return object; }
+
+        *current = SUOBJECT((*current)).mParent;
+    }
+}
+
+
 char** readline_on_complete(const char* text, int start, int end)
 {
     stack_start_stack();
@@ -407,27 +492,51 @@ char** readline_on_complete(const char* text, int start, int end)
         else {
             sCommand* command = statment->mCommands + statment->mCommandsNum-1;
 
-            const int command_num = command->mArgsNum + (SBLOCK(gReadlineBlock).mCompletionFlags & COMPLETION_FLAGS_FILE_COMPLETION ? 1:0);
+            const int command_num = command->mArgsNum + (SBLOCK(gReadlineBlock).mCompletionFlags & COMPLETION_FLAGS_WORDBREAK_CHARACTER ? 1:0);
 
             sObject* ucompletion;
             if(command->mArgsNum > 0) {
-                sObject* object = gCompletionObject;
-                int i;
-                for(i=0; i<command->mMessagesNum; i++) {
-                    object = hash_item(object, command->mMessages[i]);
-                    if(object == NULL || TYPE(object) != T_UOBJECT) {
-                        break;
-                    }
-                }
-                if(object && TYPE(object) == T_UOBJECT) {
-                    ucompletion = hash_item(object, command->mArgs[0]);
+                sObject* completion_object = gCompletionObject;
+                sObject* current_object = gCurrentObject;
 
-                    if(ucompletion == NULL) {
-                        ucompletion = hash_item(object, "*");
+                get_current_completion_object(&completion_object, &current_object);
+
+                if(completion_object == NULL) completion_object = gCompletionObject;
+
+                sObject* object;
+                if(command->mMessagesNum > 0) {
+                    sObject* reciever = completion_object;
+                    object = access_object_compl(command->mMessages[0], &reciever);
+
+                    int i;
+                    for(i=1; i<command->mMessagesNum; i++) {
+                        if(object && TYPE(object) == T_UOBJECT) {
+                            object = hash_item(object, command->mMessages[i]);
+                        }
+                        else {
+                            break;
+                        }
+                    }
+
+                    if(object && TYPE(object) == T_UOBJECT) {
+                        ucompletion = hash_item(object, command->mArgs[0]);
+
+                        if(ucompletion == NULL) {
+                            ucompletion = hash_item(object, "__all__");;
+                        }
+                    }
+                    else {
+                        ucompletion = NULL;
                     }
                 }
                 else {
-                    ucompletion = NULL;
+                    sObject* reciever = completion_object;
+                    ucompletion = access_object_compl(command->mArgs[0], &reciever);
+
+                    if(ucompletion == NULL) {
+                        reciever = completion_object;
+                        ucompletion = access_object_compl("__all__", &reciever);
+                    }
                 }
             }
             else {
@@ -466,9 +575,23 @@ char** readline_on_complete(const char* text, int start, int end)
                 SFUN(fun).mLocalObjects = stackframe;
 
                 sObject* argv = VECTOR_NEW_GC(16, FALSE);
-                if(command->mArgsNum > 0) {
+                if(command->mArgsNum == 1 || (SBLOCK(gReadlineBlock).mCompletionFlags & COMPLETION_FLAGS_WORDBREAK_CHARACTER)) 
+                {
                     vector_add(argv, STRING_NEW_GC(command->mArgs[0], FALSE));
-                    vector_add(argv, STRING_NEW_GC(command->mArgs[command->mArgsNum-1], FALSE));
+                    vector_add(argv, STRING_NEW_GC("", FALSE));
+                }
+                else if(command->mArgsNum > 1) {
+                    vector_add(argv, STRING_NEW_GC(command->mArgs[0], FALSE));
+
+                    /// if parser uses PARSER_MAGIC_NUMBER_OPTION, convert it
+                    char* str = command->mArgs[command->mArgsNum-1];
+                    char* new_str = MALLOC(strlen(str) + 1);
+                    xstrncpy(new_str, str, strlen(str) + 1);
+                    if(new_str[0] == PARSER_MAGIC_NUMBER_OPTION) {
+                        new_str[0] = '-';
+                    }
+                    vector_add(argv, STRING_NEW_GC(new_str, FALSE));
+                    FREE(new_str);
                 }
                 else {
                     vector_add(argv, STRING_NEW_GC("", FALSE));
@@ -491,12 +614,10 @@ char** readline_on_complete(const char* text, int start, int end)
 
                 fd_split(nextout, kLF);
 
-                if(TYPE(ucompletion) == T_COMPLETION) {
-                    gUserCompletionNextout = nextout;
-                    char** result = rl_completion_matches(text, user_completion);
-                    stack_end_stack();
-                    return result;
-                }
+                gUserCompletionNextout = nextout;
+                char** result = rl_completion_matches(text, user_completion);
+                stack_end_stack();
+                return result;
             }
         }
     }
@@ -505,47 +626,6 @@ char** readline_on_complete(const char* text, int start, int end)
     return NULL;
 }
 
-static int skip_quoted(const char *s, int i, char q)
-{
-   while(s[i] && s[i]!=q) 
-   {
-      if(s[i]=='\\' && s[i+1]) i++;
-      i++;
-   }
-   if(s[i]) i++;
-   return i;
-}
-
-static int lftp_char_is_quoted(const char *string,int eindex)
-{
-  int i, pass_next;
-
-  for (i = pass_next = 0; i <= eindex; i++)
-    {
-      if (pass_next)
-        {
-          pass_next = 0;
-          if (i >= eindex)
-            return 1;
-          continue;
-        }
-      else if (string[i] == '"' || string[i] == '\'')
-        {
-          char quote = string[i];
-          i = skip_quoted (string, ++i, quote);
-          if (i > eindex)
-            return 1;
-          i--;  /* the skip functions increment past the closing quote. */
-        }
-      else if (string[i] == '\\')
-        {
-          pass_next = 1;
-          continue;
-        }
-    }
-  return (0);
-}
-
 BOOL cmd_completion(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sCommand* command = runinfo->mCommand;
@@ -562,7 +642,7 @@ BOOL cmd_completion(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             while(*p) {
                 if(*p == '-' && *(p+1) == '>') {
                     if(string_c_str(str)[0] == 0) {
-                        err_msg("invalid object name", runinfo->mSName, runinfo->mSLine);
+                        err_msg("invalid object name", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
                     p+=2;
@@ -589,9 +669,7 @@ BOOL cmd_completion(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 hash_put(object, string_c_str(str), block_clone_gc(block, T_COMPLETION, FALSE));
             }
             else {
-                char buf[BUFSIZ];
-                snprintf(buf, BUFSIZ, "There is no %s object", command->mArgsRuntime[i]);
-                err_msg(buf, runinfo->mSName, runinfo->mSLine);
+                err_msg("There is no object", runinfo->mSName, runinfo->mSLine, command->mArgsRuntime[i]);
 
                 return FALSE;
             }
@@ -603,6 +681,79 @@ BOOL cmd_completion(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+BOOL cmd_readline_clear_screen(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    mclear_immediately();
+    rl_forced_update_display();
+
+    runinfo->mRCode = 0;
+
+    return TRUE;
+}
+
+BOOL cmd_readline_point_move(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+
+    if(command->mArgsNumRuntime == 2) {
+        int n = atoi(command->mArgsRuntime[1]);
+
+        if(n < 0) n += strlen(rl_line_buffer) + 1;
+        if(n < 0) n = 0;
+        if(n > strlen(rl_line_buffer)) n = strlen(rl_line_buffer);
+
+        rl_point = n;
+
+        runinfo->mRCode = 0;
+    }
+
+    return TRUE;
+}
+
+BOOL cmd_readline_insert_text(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+
+    if(runinfo->mFilter) {
+        (void)rl_insert_text(SFD(nextin).mBuf);
+#if defined(__FREEBSD__)
+//extern int rl_reset_terminal PARAMS((const char *));
+    mclear_immediately();
+    rl_forced_update_display();
+#endif
+
+        runinfo->mRCode = 0;
+    }
+
+    return TRUE;
+}
+
+BOOL cmd_readline_delete_text(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+
+    if(command->mArgsNumRuntime == 3) {
+        int n = atoi(command->mArgsRuntime[1]);
+        int m = atoi(command->mArgsRuntime[2]);
+        if(n < 0) n += strlen(rl_line_buffer) + 1;
+        if(n < 0) n= 0;
+        if(m < 0) m += strlen(rl_line_buffer) + 1;
+        if(m < 0) m = 0;
+        rl_point -= rl_delete_text(n, m);
+        if(rl_point < 0) {
+            rl_point = 0;
+        }
+#if defined(__FREEBSD__)
+    mclear_immediately();
+    rl_forced_update_display();
+#endif
+
+        runinfo->mRCode = 0;
+    }
+
+    return TRUE;
+}
+
 static int readline_macro(int count, int key)
 {
     stack_start_stack();
@@ -659,15 +810,70 @@ static int readline_macro(int count, int key)
     return 0;
 }
 
+static int skip_quoted(const char *s, int i, char q)
+{
+   while(s[i] && s[i]!=q) 
+   {
+      if(s[i]=='\\' && s[i+1]) i++;
+      i++;
+   }
+   if(s[i]) i++;
+   return i;
+}
+
+static int lftp_char_is_quoted(const char *string,int eindex)
+{
+  int i, pass_next;
+
+  for (i = pass_next = 0; i <= eindex; i++)
+    {
+      if (pass_next)
+        {
+          pass_next = 0;
+          if (i >= eindex)
+            return 1;
+          continue;
+        }
+      else if (string[i] == '"' || string[i] == '\'')
+        {
+          char quote = string[i];
+          i = skip_quoted (string, ++i, quote);
+          if (i > eindex)
+            return 1;
+          i--;  /* the skip functions increment past the closing quote. */
+        }
+      else if (string[i] == '\\')
+        {
+          pass_next = 1;
+          continue;
+        }
+    }
+  return (0);
+}
+
+char* readline_filename_completion_null_generator(const char* a, int b)
+{
+    return NULL;
+}
+
 void pipes_readline_init(BOOL runtime_script)
 {
     rl_attempted_completion_function = readline_on_complete;
+    rl_completion_entry_function = readline_filename_completion_null_generator;
     rl_completer_quote_characters = "\"'";
-    rl_completer_word_break_characters = " \t\n\"'|!&;()=";
-    rl_filename_quote_characters = " \\\t\n\"'|$*?[]{}&;#()";
+    rl_completer_word_break_characters = " \t";
     rl_completion_append_character= ' ';
     rl_char_is_quoted_p = (rl_linebuf_func_t*)lftp_char_is_quoted;
 
+//rl_basic_word_break_characters = " ";
+//      "\"'",		      // rl_completer_quote_characters
+//      " \t\n\"'",	      // rl_completer_word_break_characters
+    //rl_filename_quote_characters = " ";
+    //rl_completer_quote_characters = " \\\t\n\"'|$*?[]{}&;#()";
+    //rl_completer_quote_characters = "\"'";
+    //rl_completer_word_break_characters = " \t\n\"'|!&;()=";
+    //rl_filename_quote_characters = " \\\t\n\"'|$*?[]{}&;#()";
+
     rl_bind_key('x'-'a'+1, readline_macro);
 }
 
