@@ -340,13 +340,13 @@ void job_delete_gc(sObject* self)
     if(SJOB(self).mName) FREE(SJOB(self).mName);
 }
 
-static BOOL nextout_reader(sObject* nextout, int* pipeoutfds, sRunInfo* runinfo)
+static BOOL nextout_reader(sObject* nextout, int* pipeoutfds, sRunInfo* runinfo, char* program)
 {
     //if(!gPipesSigUser) {
         char buf[BUFSIZ+1];
         while(1) {
             if(gPipesSigInt) {
-                err_msg("signal interrupt1", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt1", runinfo->mSName, runinfo->mSLine, program);
                 gPipesSigInt = FALSE;
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
@@ -355,7 +355,7 @@ static BOOL nextout_reader(sObject* nextout, int* pipeoutfds, sRunInfo* runinfo)
             if(r < 0) {
                 if(errno == EINTR) {
                     if(!fd_write(nextout, buf)) {
-                        err_msg("signal interrupt2", runinfo->mSName, runinfo->mSLine);
+                        err_msg("signal interrupt2", runinfo->mSName, runinfo->mSLine, program);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
@@ -369,7 +369,7 @@ static BOOL nextout_reader(sObject* nextout, int* pipeoutfds, sRunInfo* runinfo)
             buf[r] = 0;
 
             if(!fd_write(nextout, buf)) {
-                err_msg("signal interrupt3", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt3", runinfo->mSName, runinfo->mSLine, program);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
@@ -385,7 +385,7 @@ static BOOL nextout_reader(sObject* nextout, int* pipeoutfds, sRunInfo* runinfo)
     return TRUE;
 }
 
-static BOOL nextin_writer(pid_t pid, sObject* nextin, int* pipeinfds, int* pipeoutfds, int* pipeerrfds, sRunInfo* runinfo)
+static BOOL nextin_writer(pid_t pid, sObject* nextin, int* pipeinfds, int* pipeoutfds, int* pipeerrfds, sRunInfo* runinfo, char* program)
 {
     int pid2 = fork();
     if(pid2 < 0) {
@@ -442,7 +442,7 @@ static BOOL nextin_writer(pid_t pid, sObject* nextin, int* pipeinfds, int* pipeo
                 kill(pid2, SIGCONT);
             }
             else if(WIFSIGNALED(status)) {
-                err_msg("signal interrupt4", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt4", runinfo->mSName, runinfo->mSLine, program);
                 return FALSE;
             }
             else {
@@ -456,7 +456,7 @@ static BOOL nextin_writer(pid_t pid, sObject* nextin, int* pipeinfds, int* pipeo
 
         if(WIFSIGNALED(status) || WIFSTOPPED(status))
         {
-            err_msg("signal interrupt5", runinfo->mSName, runinfo->mSLine);
+            err_msg("signal interrupt5", runinfo->mSName, runinfo->mSLine, program);
             return FALSE;
         }
     }
@@ -464,7 +464,7 @@ static BOOL nextin_writer(pid_t pid, sObject* nextin, int* pipeinfds, int* pipeo
     return TRUE;
 }
 
-static BOOL wait_child_program(pid_t pid, int nextout2, sRunInfo* runinfo)
+static BOOL wait_child_program(pid_t pid, int nextout2, sRunInfo* runinfo, char* program)
 {
     if(!runinfo->mLastProgram) {
         if(gAppType == kATOptC) {
@@ -478,7 +478,7 @@ static BOOL wait_child_program(pid_t pid, int nextout2, sRunInfo* runinfo)
                     kill(pid, SIGCONT);
                 }
                 else if(WIFSIGNALED(status)) {
-                    err_msg("signal interrupt6", runinfo->mSName, runinfo->mSLine);
+                    err_msg("signal interrupt6", runinfo->mSName, runinfo->mSLine, program);
                     return FALSE;
                 }
                 else {
@@ -491,7 +491,7 @@ static BOOL wait_child_program(pid_t pid, int nextout2, sRunInfo* runinfo)
             pid = waitpid(pid, &status, WUNTRACED);
 
             if(WIFSTOPPED(status)) {
-                err_msg("signal interrupt7", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt7", runinfo->mSName, runinfo->mSLine, program);
 
                 kill(pid, SIGKILL);
                 pid = waitpid(pid, &status, WUNTRACED);
@@ -502,7 +502,7 @@ static BOOL wait_child_program(pid_t pid, int nextout2, sRunInfo* runinfo)
                 return FALSE;
             }
             else if(WIFSIGNALED(status)) {  // a pipes external program which is not a last command can't be stopped by CTRL-Z
-                err_msg("signal interrupt8", runinfo->mSName, runinfo->mSLine);
+                err_msg("signal interrupt8", runinfo->mSName, runinfo->mSLine, program);
 
                 if(tcsetpgrp(0, getpid()) < 0) {
                     perror("tcsetpgrp(pipes)");
@@ -631,6 +631,26 @@ static BOOL run_exec_cprog(sCommand* command, char* program, int nextin, int nex
     }
 
     /// exec ///
+    BOOL program_at_cd = TRUE;
+    char* p = program;
+    while(*p) {
+        if(*p == '/') {
+            program_at_cd = FALSE;
+            break;
+        }
+        else {
+            p++;
+        }
+    }
+
+    if(program_at_cd) {
+        if(access(program, X_OK) == 0) {
+            fprintf(stderr, "This is program at current directory\n");
+            kill(getppid(), SIGUSR1);
+            exit(1);
+        }
+    }
+
     execv(program, command->mArgsRuntime);
     //fprintf(stderr, "exec('%s') error\n", command->mArgsRuntime[0]);  // comment out for try
     kill(getppid(), SIGUSR1);
@@ -735,29 +755,29 @@ static BOOL run_external_command(char* program, sObject* nextin, sObject* nextou
         /// Therefore pipes can't waitpid and deadlock.
         /// Avoid this, pipes is ready for a writer process to write buffer to the pipes.
         if(nextin2 != 0) {
-            if(!nextin_writer(pgroup, nextin, pipeinfds, pipeoutfds, pipeerrfds, runinfo)) {
-                (void)wait_child_program(pid, nextout2, runinfo);
+            if(!nextin_writer(pgroup, nextin, pipeinfds, pipeoutfds, pipeerrfds, runinfo, program)) {
+                (void)wait_child_program(pid, nextout2, runinfo, program);
                 return FALSE;
             }
         }
         if(nextout2 != 1) {
             (void)close(pipeoutfds[1]);
-            if(!nextout_reader(nextout, pipeoutfds, runinfo)) {
+            if(!nextout_reader(nextout, pipeoutfds, runinfo, program)) {
                 // wait everytime
-                (void)wait_child_program(pid, nextout2, runinfo);
+                (void)wait_child_program(pid, nextout2, runinfo, program);
                 return FALSE;
             }
         }
         if(nexterr2 != 2) {
             (void)close(pipeerrfds[1]);
-            if(!nextout_reader(gStderr, pipeerrfds, runinfo)) {
-                (void)wait_child_program(pid, nextout2, runinfo);
+            if(!nextout_reader(gStderr, pipeerrfds, runinfo, program)) {
+                (void)wait_child_program(pid, nextout2, runinfo, program);
                 return FALSE;
             }
         }
 
         // wait everytime
-        if(!wait_child_program(pid, nextout2, runinfo)) {
+        if(!wait_child_program(pid, nextout2, runinfo, program)) {
             return FALSE;
         }
 
@@ -765,9 +785,11 @@ static BOOL run_external_command(char* program, sObject* nextin, sObject* nextou
     }
 }
 
-BOOL run_function(sObject* fun, sObject* current_object, sObject* nextin, sObject* nextout, sRunInfo* runinfo, char** argv, int argc, sObject** blocks, int blocks_num)
+BOOL run_function(sObject* fun, sObject* nextin, sObject* nextout, sRunInfo* runinfo, char** argv, int argc, sObject** blocks, int blocks_num)
 {
     sObject* local_objects = SFUN(fun).mLocalObjects;
+    sRunInfo* runinfo2 = SFUN(fun).mRunInfo;
+    SFUN(fun).mRunInfo = runinfo;
 
     sObject* stackframe = UOBJECT_NEW_GC(8, gPipesObject, "_stackframe", FALSE);
     vector_add(gStackFrames, stackframe);
@@ -805,8 +827,10 @@ BOOL run_function(sObject* fun, sObject* current_object, sObject* nextin, sObjec
     }
 
     int rcode = 0;
-    if(!run(fun, nextin, nextout, &rcode, current_object, fun)) {
+    if(!run(fun, nextin, nextout, &rcode, runinfo->mCurrentObject, fun)) {
         if(rcode != RCODE_RETURN) {
+            char buf[BUFSIZ];
+            err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = rcode;
             SFUN(fun).mLocalObjects = local_objects;
             (void)vector_pop_back(gStackFrames);
@@ -818,14 +842,17 @@ BOOL run_function(sObject* fun, sObject* current_object, sObject* nextin, sObjec
 
     SFUN(fun).mLocalObjects = local_objects;
     SFUN(fun).mArgBlocks = arg_blocks;
+    SFUN(fun).mRunInfo = runinfo2;
 
     return TRUE;
 }
 
-BOOL run_block(sObject* block, sObject* current_object, sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+static BOOL run_block(sObject* block, sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     int rcode = 0;
-    if(!run(block, nextin, nextout, &rcode, current_object, runinfo->mRunningObject)) {
+    if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
+        sCommand* command = runinfo->mCommand;
+        err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         runinfo->mRCode = rcode;
         return FALSE;
     }
@@ -833,7 +860,7 @@ BOOL run_block(sObject* block, sObject* current_object, sObject* nextin, sObject
     return TRUE;
 }
 
-static BOOL run_object(sObject* object, sObject* reciever, sObject* current_object, sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+static BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     switch(TYPE(object)) {
         case T_NFUN: 
@@ -845,8 +872,6 @@ static BOOL run_object(sObject* object, sObject* reciever, sObject* current_obje
                 }
             }
 
-            runinfo->mCurrentObject = current_object;
-            runinfo->mRecieverObject = reciever;
             runinfo->mRCode = RCODE_NFUN_INVALID_USSING;
 
             if(!SNFUN(object).mNativeFun(nextin, nextout, runinfo)) {
@@ -854,16 +879,15 @@ static BOOL run_object(sObject* object, sObject* reciever, sObject* current_obje
             }
             if(runinfo->mRCode == RCODE_NFUN_INVALID_USSING) {
                 sCommand* command = runinfo->mCommand;
-                char buf[BUFSIZ];
-                snprintf(buf, BUFSIZ, "invalid command using(%s)\n", command->mArgsRuntime[0]);
-                err_msg(buf, runinfo->mSName, runinfo->mSLine);
+                err_msg("invalid command using", runinfo->mSName, runinfo->mSLine, command->mArgsRuntime[0]);
                 return FALSE;
             }
             break;
 
         case T_FUN: {
             sCommand* command = runinfo->mCommand;
-            if(!run_function(object, reciever, nextin, nextout, runinfo, command->mArgsRuntime + 1, command->mArgsNumRuntime -1, command->mBlocks, command->mBlocksNum)){
+            runinfo->mCurrentObject = runinfo->mRecieverObject;
+            if(!run_function(object, nextin, nextout, runinfo, command->mArgsRuntime + 1, command->mArgsNumRuntime -1, command->mBlocks, command->mBlocksNum)){
                 return FALSE;
             }
             }
@@ -871,7 +895,7 @@ static BOOL run_object(sObject* object, sObject* reciever, sObject* current_obje
 
         case T_CLASS: {
             sCommand* command = runinfo->mCommand;
-            if(!run_function(object, current_object, nextin, nextout, runinfo, command->mArgsRuntime + 1, command->mArgsNumRuntime -1, command->mBlocks, command->mBlocksNum)){
+            if(!run_function(object, nextin, nextout, runinfo, command->mArgsRuntime + 1, command->mArgsNumRuntime -1, command->mBlocks, command->mBlocksNum)){
                 return FALSE;
             }
             }
@@ -884,19 +908,21 @@ static BOOL run_object(sObject* object, sObject* reciever, sObject* current_obje
             break;
 
         case T_BLOCK:
-            if(!run_block(object, current_object, nextin, nextout, runinfo)) {
+            if(!run_block(object, nextin, nextout, runinfo)) {
                 return FALSE;
             }
             break;
 
         case T_STRING: {
             if(!fd_write(nextout, string_c_str(object))) {
-                err_msg("signal interrupt9", runinfo->mSName, runinfo->mSLine);
+                sCommand* command = runinfo->mCommand;
+                err_msg("signal interrupt9", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
             if(!fd_write(nextout, "\n")) {
-                err_msg("signal interrupt10", runinfo->mSName, runinfo->mSLine);
+                sCommand* command = runinfo->mCommand;
+                err_msg("signal interrupt10", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
@@ -904,9 +930,18 @@ static BOOL run_object(sObject* object, sObject* reciever, sObject* current_obje
             break;
 
         case T_UOBJECT: {
-            runinfo->mCurrentObject = object;
-            if(!cmd_mshow(nextin, nextout, runinfo)) {
-                return FALSE;
+            sObject* main_obj = hash_item(object, "main");
+            runinfo->mRecieverObject = object;
+
+            if(main_obj) {
+                if(!run_object(main_obj, nextin, nextout, runinfo)) {
+                    return FALSE;
+                }
+            }
+            else {
+                if(!cmd_mshow(nextin, nextout, runinfo)) {
+                    return FALSE;
+                }
             }
             }
             break;
@@ -916,7 +951,7 @@ static BOOL run_object(sObject* object, sObject* reciever, sObject* current_obje
             for(j=0; j<vector_size(object); j++) {
                 sObject* item = vector_item(object, j);
 
-                if(!run_object(item, reciever, current_object, nextin, nextout, runinfo)) {
+                if(!run_object(item, nextin, nextout, runinfo)) {
                     return FALSE;
                 }
             }
@@ -928,7 +963,7 @@ static BOOL run_object(sObject* object, sObject* reciever, sObject* current_obje
             while(it) {
                 sObject* item = hash_loop_item(it);
 
-                if(!run_object(item, reciever, current_object, nextin, nextout, runinfo)) {
+                if(!run_object(item, nextin, nextout, runinfo)) {
                     return FALSE;
                 }
 
@@ -939,8 +974,8 @@ static BOOL run_object(sObject* object, sObject* reciever, sObject* current_obje
 
         default: {
             char buf[BUFSIZ];
-            snprintf(buf, BUFSIZ, "can't run this object(%s)", runinfo->mCommand->mArgsRuntime[0]);
-            err_msg(buf, runinfo->mSName, runinfo->mSLine);
+            sCommand* command = runinfo->mCommand;
+            err_msg("can't run this object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             }
             return FALSE;
     }
@@ -976,7 +1011,8 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
     if(statment->mFlags & STATMENT_CONTEXTPIPE) {
         if(pipein == gStdin && SFD(gStdin).mBufLen == 0) {
             if(!stdin_read_out()) {
-                err_msg("signal interrupt11", runinfo->mSName, runinfo->mSLine);
+                sCommand* command = runinfo->mCommand;
+                err_msg("signal interrupt11", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
@@ -987,7 +1023,8 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
     else if(statment->mFlags & STATMENT_GLOBALPIPEIN) {
         nextin = FD_NEW_STACK();
         if(!fd_write(nextin, SFD(gGlobalPipe).mBuf)) {
-            err_msg("signal interrupt12", runinfo->mSName, runinfo->mSLine);
+            sCommand* command = runinfo->mCommand;
+            err_msg("signal interrupt12", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
@@ -1020,7 +1057,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
 
         if(command->mArgsNum > 0) {
             if(command->mArgsEnv[0]) {
-                err_msg("a command name must be determined staticaly. Instead of this, use eval inner command.", runinfo->mSName, runinfo->mSLine);
+                err_msg("a command name must be determined staticaly. Instead of this, use eval inner command.", runinfo->mSName, runinfo->mSLine, NULL);
                 return FALSE;
             }
 
@@ -1035,7 +1072,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                         object = hash_item(object, command->mMessages[i++]);
                     }
                     else {
-                        err_msg("invalid message passing", runinfo->mSName, runinfo->mSLine);
+                        err_msg("invalid message passing", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
                 }
@@ -1051,19 +1088,18 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                     }
 
                     if(object) {
-                        if(!run_object(object, reciever, current_object, nextin, nextout, runinfo)) {
+                        runinfo->mRecieverObject = reciever;
+                        if(!run_object(object, nextin, nextout, runinfo)) {
                             return FALSE;
                         }
                     }
                     else {
-                        char buf[BUFSIZ];
-                        snprintf(buf, BUFSIZ, "there is not this object(%s)", command->mArgsRuntime[0]);
-                        err_msg(buf, runinfo->mSName, runinfo->mSLine);
+                        err_msg("there is not this object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
                 }
                 else {
-                    err_msg("invalid message passing", runinfo->mSName, runinfo->mSLine);
+                    err_msg("invalid message passing", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     return FALSE;
                 }
             }
@@ -1079,7 +1115,8 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                         return FALSE;
                     }
 
-                    if(!run_object(object, reciever, current_object, nextin, nextout, runinfo)) {
+                    runinfo->mRecieverObject = reciever;
+                    if(!run_object(object, nextin, nextout, runinfo)) {
                         return FALSE;
                     }
                 }
@@ -1145,7 +1182,8 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
         /// flash stdout and stderr ///
         if(pipeout == gStdout) {
             if(!fd_flash(gStdout, 1)) {
-                err_msg("signal interrupt13", runinfo.mSName, runinfo.mSLine);
+                sCommand* command = runinfo.mCommand;
+                err_msg("signal interrupt13", runinfo.mSName, runinfo.mSLine, command->mArgs[0]);
                 *rcode = RCODE_SIGNAL_INTERRUPT;
                 stack_end_stack();
                 nest_level--;
@@ -1155,7 +1193,8 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
 
         if(nest_level == 1) {
             if(!fd_flash(gStderr, 2)) {
-                err_msg("signal interrupt14", runinfo.mSName, runinfo.mSLine);
+                sCommand* command = runinfo.mCommand;
+                err_msg("signal interrupt14", runinfo.mSName, runinfo.mSLine, command->mArgs[0]);
                 *rcode = RCODE_SIGNAL_INTERRUPT;
                 stack_end_stack();
                 nest_level--;
@@ -1166,14 +1205,16 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
         /// the end of statment ///
         if(gPipesSigUser) {
             gPipesSigUser = FALSE;
-            err_msg("command not found", runinfo.mSName, runinfo.mSLine);
+            sCommand* command = runinfo.mCommand;
+            err_msg("command not found", runinfo.mSName, runinfo.mSLine, ""); //command->mArgs[0]);
             stack_end_stack();
             nest_level--;
             return FALSE;
         }
         else if(runinfo.mRCode == 128+SIGINT || gPipesSigInt) {
             gPipesSigInt = FALSE;
-            err_msg("signal interrupt15", runinfo.mSName, runinfo.mSLine);
+            sCommand* command = runinfo.mCommand;
+            err_msg("signal interrupt15", runinfo.mSName, runinfo.mSLine, command->mArgs[0]);
             *rcode = RCODE_SIGNAL_INTERRUPT;
             stack_end_stack();
             nest_level--;
@@ -1272,7 +1313,7 @@ BOOL forground_job(int num)
         return TRUE;
     }
 
-    err_msg("invalid job number", "fg", 1);
+    err_msg("invalid job number", "fg", 1, "fg");
     return FALSE;
 }
 
@@ -1367,16 +1408,15 @@ BOOL load_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo
 {
     int fd = open(fname, O_RDONLY);
     if(fd < 0) {
-        char buf[BUFSIZ];
-        snprintf(buf, BUFSIZ, "open error(%s)", fname);
-        err_msg(buf, runinfo->mSName, runinfo->mSLine);
+        err_msg("open error", runinfo->mSName, runinfo->mSLine, fname);
         return FALSE;
     }
 
     char* buf;
     int buf_size;
     if(!bufsiz_read(fd, ALLOC &buf, &buf_size)) {
-        err_msg("signal interrupt16", runinfo->mSName, runinfo->mSLine);
+        sCommand* command = runinfo->mCommand;
+        err_msg("signal interrupt16", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         FREE(buf);
         close(fd);
@@ -1409,7 +1449,6 @@ BOOL load_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo
             if(rcode == RCODE_EXIT) {
             }
             else {
-                err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine);
                 stack_end_stack();
                 (void)vector_pop_back(gStackFrames);
                 return FALSE;
@@ -1418,7 +1457,8 @@ BOOL load_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo
     }
     else {
         FREE(buf);
-        err_msg_adding("parser error", runinfo->mSName, runinfo->mSLine);
+        sCommand* command = runinfo->mCommand;
+        err_msg_adding("parser error", runinfo->mSName, runinfo->mSLine, "parser");
         stack_end_stack();
         (void)vector_pop_back(gStackFrames);
         return FALSE;
